@@ -9,59 +9,12 @@ use Scalar::Util qw( blessed );
 use Storable qw( dclone );
 use Carp qw( croak );
 
-our $VERSION = '0.05003';
+our $VERSION = '0.08002';
 $VERSION = eval $VERSION;
 
-# sub _compatible_config() is only required as long as we support deprecated
-# features and can be factored out when we remove support
-
-sub _compatible_config {
-    my ($object) = @_;
-
-    return _compatible_attrs( $object->model_config );
-}
-
-sub _compatible_attrs {
-    my ($config) = @_;
-
-    return {} unless keys %$config;
-
-# legacy model_config->{DBIC} style
-# test this works by reverting the test suite back `svn up -r 1020 t`
-# before running the tests
-
-    if ( exists $config->{DBIC} ) {
-        warn
-            "model_config->{DBIC}{} is deprecated and is provided for compatibilty only\n"
-            . "and will be removed: use model_config->{} instead.";
-
-        $config = dclone($config);
-
-        my $dbic = delete $config->{DBIC};
-
-        return _merge_hashes( $config, $dbic );
-    }
-
-    if ( exists $config->{new_empty_row} ) {
-        warn "'new_empty_row' is deprecated and is provided for compatibility only,\n"
-             . "and will be removed: see the documentation for 'empty_rows' instead.";
-    }
-
-    if ( exists $config->{new_empty_row_multi} ) {
-        warn "'new_empty_row_multi' is deprecated and is provided for compatibility only,\n"
-            . "and will be removed: see the documentation for 'new_rows_max' instead.";
-
-        $config->{new_empty_row} ||= $config->{new_empty_row_multi};
-    }
-
-    return $config;
-}
-
 sub options_from_model {
     my ( $self, $base, $attrs ) = @_;
 
-    $attrs = $attrs ? _compatible_attrs($attrs) : {};
-
     my $form      = $base->form;
     my $resultset = _get_resultset( $base, $form, $attrs );
     my $source    = $resultset->result_source;
@@ -86,6 +39,7 @@ sub options_from_model {
     $label_col = $id_col if !defined $label_col;
 
     if ( defined( my $from_stash = $attrs->{condition_from_stash} ) ) {
+        $condition = $condition ? { %{$condition} } : {}; # avoid overwriting attrs->{condition}
         for my $name ( keys %$from_stash ) {
             my $value = $form->stash->{ $from_stash->{$name} };
 
@@ -95,6 +49,8 @@ sub options_from_model {
             $condition->{$name} = $value;
         }
     }
+    # save the expanded condition for later use
+    $attrs->{'-condition'} = $condition if ($condition);
 
     $attributes->{'-columns'} = [ $id_col, $label_col ];
 
@@ -153,8 +109,6 @@ sub _get_resultset {
 sub default_values {
     my ( $self, $dbic, $attrs ) = @_;
 
-    $attrs = $attrs ? _compatible_attrs($attrs) : {};
-
     my $form = $self->form;
     my $base = defined $attrs->{base} ? delete $attrs->{base} : $form;
 
@@ -186,7 +140,7 @@ sub _fill_in_fields {
     my ( $base, $dbic ) = @_;
     for my $field ( @{ $base->get_fields } ) {
         my $name   = $field->name;
-        my $config = _compatible_config($field);
+        my $config = $field->model_config;
 
         next if not defined $name || $config->{accessor};
         next if not is_direct_child( $base, $field );
@@ -264,7 +218,7 @@ sub _fill_nested {
         next if $block->is_field && !$block->is_block;
         next if !$block->can('nested_name');
 
-        my $config = _compatible_config($block);
+        my $config = $block->model_config;
 
         # first handle {label}
 
@@ -308,7 +262,6 @@ sub _fill_nested {
             my @rows = $dbic->$rel->all;
 
             my $count = $config->{empty_rows}    ? scalar @rows + $config->{empty_rows}
-                      : $config->{new_empty_row} ? scalar @rows + 1
                       :                            scalar @rows;
 
             my $blocks = $block->repeat($count);
@@ -334,7 +287,7 @@ sub _fill_nested {
 
             # remove 'delete' checkbox from the last repetition ?
 
-            if ( $config->{empty_rows} || $config->{new_empty_row} ) {
+            if ( $config->{empty_rows} ) {
 
                 my $new_row_count = $config->{empty_rows} ? $config->{empty_rows}
                                   : 1;
@@ -346,7 +299,7 @@ sub _fill_nested {
                     my $rep = $reps[$i];
 
                     my ($del_field)
-                        = grep { _compatible_config($_)->{delete_if_true} }
+                        = grep { $_->model_config->{delete_if_true} }
                         @{ $rep->get_fields };
 
                     if ( defined $del_field ) {
@@ -369,8 +322,6 @@ sub create {
 
     croak "invalid arguments" if @_ > 2;
 
-    $attrs = $attrs ? _compatible_attrs($attrs) : {};
-
     my $form = $self->form;
     my $base = defined $attrs->{base} ? delete $attrs->{base} : $form;
 
@@ -379,8 +330,8 @@ sub create {
 
     my $resultset
         = $attrs->{resultset}
-        || _compatible_config($base)->{resultset}
-        || _compatible_config($form)->{resultset}
+        || $base->model_config->{resultset}
+        || $form->model_config->{resultset}
         or croak 'could not find resultset name';
 
     $resultset = $schema->resultset($resultset);
@@ -395,8 +346,6 @@ sub update {
 
     croak "row object missing" if !defined $dbic;
 
-    $attrs = $attrs ? _compatible_attrs($attrs) : $attrs;
-
     my $form = $self->form;
     my $base = defined $attrs->{base} ? delete $attrs->{base} : $form;
 
@@ -477,6 +426,8 @@ sub _save_relationships {
 
         }
         elsif ( defined $block && ref $params eq 'HASH' ) {
+            $dbic->discard_changes unless( $dbic->$rel );
+            
             my $target = $dbic->find_related( $rel, {} );
 
             if ( !defined $target && grep { length $_ } values %$params ) {
@@ -492,6 +443,10 @@ sub _save_relationships {
                     nested_base => $rel,
                     from        => $dbic->result_class,
                 } );
+            unless($dbic->$rel) {
+                $dbic->$rel($target);
+                $dbic->update;
+            }
         }
         elsif ( defined $multi_value ) {
             # belongs_to, has_one or might_have relationship
@@ -528,7 +483,7 @@ sub _save_relationships {
             # this is supposed to indicate a has_one/might_have...
             # where's the introspection!!?? :)
             else {
-                $fk_constraint = not $dbic->result_source->compare_relationship_keys( \@keys, \@fpkey );
+                $fk_constraint = not _compare_relationship_keys( \@keys, \@fpkey );
             }
 
             next if($fk_constraint);
@@ -565,6 +520,41 @@ sub _save_relationships {
     }
 }
 
+# Copied from DBIx::Class::ResultSource
+sub _compare_relationship_keys {
+  my ($keys1, $keys2) = @_;
+
+  # Make sure every keys1 is in keys2
+  my $found;
+  foreach my $key (@$keys1) {
+    $found = 0;
+    foreach my $prim (@$keys2) {
+      if ($prim eq $key) {
+        $found = 1;
+        last;
+      }
+    }
+    last unless $found;
+  }
+
+  # Make sure every key2 is in key1
+  if ($found) {
+    foreach my $prim (@$keys2) {
+      $found = 0;
+      foreach my $key (@$keys1) {
+        if ($prim eq $key) {
+          $found = 1;
+          last;
+        }
+      }
+      last unless $found;
+    }
+  }
+
+  return $found;
+}
+
+
 sub _save_has_many {
     my ( $self, $dbic, $form, $rs, $block, $rel, $attrs ) = @_;
 
@@ -580,7 +570,7 @@ sub _save_has_many {
 
     my @blocks = @{ $block->get_elements };
     my $max    = $#blocks;
-    my $config = _compatible_config($block);
+    my $config = $block->model_config;
 
     my $new_rows_max     = $config->{new_rows_max} || $config->{empty_rows} || 0;
     my $new_rows_counter = 0;
@@ -604,16 +594,9 @@ sub _save_has_many {
 
         if (   ( !defined $value || $value eq '' )
             && (
-                ( # either new behaviour
-                    $new_rows_max
-                    && ( ++$new_rows_counter <= $new_rows_max )
-                )
-                ||
-                ( # or deprecated behaviour
-                    $config->{new_empty_row} )
-                    && ( $i == $max || $config->{new_empty_row_multi} )
-                )
-            )
+                $new_rows_max
+                && ( ++$new_rows_counter <= $new_rows_max )
+            ) )
         {
 
             # insert a new row
@@ -656,69 +639,45 @@ sub _insert_has_many {
 sub _can_insert_new_row {
     my ( $dbic, $form, $config, $repetition, $rel, $pk_field ) = @_;
     
-    if ( $config->{new_empty_row} ) {
-        # old, deprecated behaviour
+    my @rep_fields = @{ $repetition->get_fields };
+    
+    my $pk_name = $pk_field->nested_name;
+    
+    my @constraints = grep { $_->when->{field} eq $pk_name }
+                      grep { defined $_->when }
+                      map { @{ $_->get_constraints({ type => 'Required' }) } }
+                        @rep_fields;
+    
+    my @required_fields;
+    
+    if ( @constraints ) {
+        # if there are any Required constraints whose 'when' clause points to
+        # the PK field - check that all these fields are filled in - as
+        # the PK value is missing on new reps, so the constraint won't have run
         
-        my $rows = $config->{new_empty_row};
-    
-        $rows = [$rows] if ref $rows ne 'ARRAY';
-    
-        for my $name (@$rows) {
-            my ($field)
-                = grep { $_->original_name eq $name } @{ $repetition->get_fields };
-    
-            return if !defined $field;
-    
-            my $nested_name = $field->nested_name;
-            return if !$form->valid($nested_name);
-    
-            my $value = $form->param_value($nested_name);
-            return if !length $value;
-        }
+        return if
+            notall { defined && length }
+            map { $form->param_value( $_->nested_name ) }
+            map { $_->parent }
+              @constraints;
     }
     else {
-        # new behaviour
+        # otherwise, just check at least 1 field that matches either a column
+        # name or an accessor, is filled in
         
-        my @rep_fields = @{ $repetition->get_fields };
+        my $result_source = $dbic->$rel->result_source;
         
-        my $pk_name = $pk_field->nested_name;
+        #  only create a new record if (read from bottom)...
         
-        my @constraints = grep { $_->when->{field} eq $pk_name }
-                          grep { defined $_->when }
-                          map { @{ $_->get_constraints({ type => 'Required' }) } }
-                            @rep_fields;
-        
-        my @required_fields;
-        
-        if ( @constraints ) {
-            # if there are any Required constraints whose 'when' clause points to
-            # the PK field - check that all these fields are filled in - as
-            # the PK value is missing on new reps, so the constraint won't have run
-            
-            return if
-                notall { defined && length }
-                map { $form->param_value( $_->nested_name ) }
-                map { $_->parent }
-                  @constraints;
-        }
-        else {
-            # otherwise, just check at least 1 field that matches either a column
-            # name or an accessor, is filled in
-            
-            my $result_source = $dbic->$rel->result_source;
-            
-            #  only create a new record if (read from bottom)...
-            
-            return if
-                none { defined && length }
-                map { $form->param_value( $_->nested_name ) }
-                grep {
-                    $result_source->has_column( $_->original_name )
-                    || $result_source->can( $_->original_name )
-                }
-                grep { defined $_->original_name }
-                    @rep_fields;
-        }
+        return if
+            none { defined && length }
+            map { $form->param_value( $_->nested_name ) }
+            grep {
+                $result_source->has_column( $_->original_name )
+                || $result_source->can( $_->original_name )
+            }
+            grep { defined $_->original_name }
+                @rep_fields;
     }
     
     return 1;
@@ -728,7 +687,7 @@ sub _delete_has_many {
     my ( $form, $row, $rep ) = @_;
 
     my ($del_field)
-        = grep { _compatible_config($_)->{delete_if_true} }
+        = grep { $_->model_config->{delete_if_true} }
         @{ $rep->get_fields };
 
     return if !defined $del_field;
@@ -753,7 +712,7 @@ sub _fix_value {
     
     if ( defined $value ) {
         if ( (     $is_nullable
-                || $data_type =~ m/^timestamp|date|int|float|numeric/i
+                && $data_type =~ m/^timestamp|date|int|float|numeric/i
             )
 
             # comparing to '' does not work for inflated objects
@@ -782,7 +741,7 @@ sub _save_columns {
     for my $field ( @{ $base->get_fields }, ) {
         next if not is_direct_child( $base, $field );
         
-        my $config = _compatible_config($field);
+        my $config = $field->model_config;
         next if $config->{delete_if_true};
         next if $config->{read_only};
         
@@ -883,7 +842,7 @@ sub _save_multi_value_fields_many_to_many {
             my @values = $form->param_list($nested_name);
             my @rows;
 
-            my $config = _compatible_config($field);
+            my $config = $field->model_config;
 
             my ($pk) = $config->{default_column}
                 || $related->result_source->primary_columns;
@@ -898,9 +857,6 @@ sub _save_multi_value_fields_many_to_many {
             }
 
             if($config->{additive}) {
-                
-                my $relinfo = $dbic->result_source->relationship_info($name);
-                
                 $pk =~ s/^.*\.//;
                 
                 my $set_method = "add_to_$name";
@@ -911,9 +867,23 @@ sub _save_multi_value_fields_many_to_many {
                     $dbic->$set_method($row, $config->{link_values});
                 }
             } else {
-                my $set_method = "set_$name";
-                
-                $dbic->$set_method( \@rows, $config->{link_values} );
+                # check if there is a restricting condition on here
+                # if so life is more complex
+                my $condition = $config->{'-condition'};
+                if ($condition) {
+                    my $set_method    = "add_to_$name";
+                    my $remove_method = "remove_from_$name";
+                    foreach ( $dbic->$name->search($condition)->all ) {
+                        $dbic->$remove_method($_);
+                    }
+                    foreach my $row (@rows) {
+                        $dbic->$set_method( $row, $config->{link_values} );
+                    }
+                }
+                else {
+                    my $set_method = "set_$name";
+                    $dbic->$set_method( \@rows, $config->{link_values} );
+                }
             }
         }
     }
@@ -959,21 +929,15 @@ sub _save_repeatable_many_to_many {
                 my $row;
                 my $is_new;
 
-                my $config           = _compatible_config($block);
+                my $config           = $block->model_config;
                 my $new_rows_max     = $config->{new_rows_max} || $config->{empty_rows} || 0;
                 my $new_rows_counter = 0;
 
                 if (   ( !defined $value || $value eq '' )
                     && (
-                        ( # either new behaviour
                             $new_rows_max
                             && ( ++$new_rows_counter <= $new_rows_max )
                         )
-                        ||
-                        ( # or deprecated behaviour
-                            $config->{new_empty_row} )
-                            && $i == $max
-                        )
                     )
                 {
 
@@ -1034,7 +998,7 @@ sub _delete_many_to_many {
     my ( $form, $dbic, $row, $rel, $rep ) = @_;
 
     my ($del_field)
-        = grep { _compatible_config($_)->{delete_if_true} }
+        = grep { $_->model_config->{delete_if_true} }
         @{ $rep->get_fields };
 
     return if !defined $del_field;
@@ -1400,7 +1364,7 @@ L<multiple|HTML::FormFu::Element::Select/multiple> set.
 The field's L<name|HTML::FormFu::Element::_Field/name> must be set to the 
 name of the C<many_to_many> relationship.
 
-=item default_column
+=head3 default_column
 
 If you want to search / associate the related table by a column other it's
 primary key, set C<< $field->model_config->{default_column} >>.
@@ -1412,6 +1376,8 @@ primary key, set C<< $field->model_config->{default_column} >>.
           model_config:
             default_column: foo
 
+=head3 link_values
+
 If you want to set columns on the link table you can do so if you add a 
 C<link_values> attribute to C<model_config>:
 
@@ -1423,6 +1389,7 @@ C<link_values> attribute to C<model_config>:
             link_values:
               foo: bar
 
+=head3 additive
 
 The default implementation will first remove all related objects and set the
 new ones (see L<http://search.cpan.org/perldoc?DBIx::Class::Relationship::Base#set_$rel>).
@@ -1437,7 +1404,7 @@ set C<additive> in the C<model_config>.
             additive: 1
             options_from_model: 0
 
-(<options_from_model> is set to C<0> because this L</options_from_model> will try to fetch
+L</options_from_model> is set to C<0> because it will try to fetch
 all objects from the result class C<Authors> if C<model_config> is specified
 without a C<resultset> attribute.)
 
@@ -1648,7 +1615,7 @@ C<localize_label>
         model_config:
           localize_label: 1
 
-You can set a C<condition>, which will be passed as the 1st arguement to
+You can set a C<condition>, which will be passed as the 1st argument to
 L<DBIx::Class::ResultSet/search>.
 
     element:
@@ -1659,9 +1626,43 @@ L<DBIx::Class::ResultSet/search>.
           condition:
             type: is_foo
 
-You can set C<attributes>, which will be passed as the 2nd arguement to
+You can set a C<condition_from_stash>, which will be passed as the 1st argument to
 L<DBIx::Class::ResultSet/search>.
 
+C<key> is the column-name to be passed to
+L<search|DBIx::Class::ResultSet/search>,
+and C<stash_key> is the name of a key on the form L<stash|HTML::FormFu/stash>
+from which the value to be passed to L<search|DBIx::Class::ResultSet/search>
+is found.
+
+    element:
+      - type: Select
+        name: foo
+        model_config:
+          resultset: TableClass
+          condition_from_stash:
+            key: stash_key
+
+Is comparable to:
+
+    $form->element({
+        type => 'Select',
+        name => 'foo',
+        model_config => {
+            resultset => 'TableClass',
+            condition => {
+                key => $form->stash->{stash_key}
+            }
+        }
+    })
+
+You can set C<attributes>, which will be passed as the 2nd argument to
+L<DBIx::Class::ResultSet/search>.
+
+
+
+
+
 =head1 FAQ
 
 =head2 Add extra values not in the form
@@ -1690,32 +1691,6 @@ the user even if they submit a value for it.
 
 See L<HTML::FormFu::Element::Label>.
 
-=head1 DEPRECATED
-
-=head2 new_empty_row
-
-Is deprecated and provided only for backwards compatability.
-Will be removed at some point in the future.
-
-See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
-
-=head2 new_empty_row_multi
-
-Is deprecated and provided only for backwards compatability.
-Will be removed at some point in the future.
-
-See C<new_rows_max> in L</"Config options for Repeatable blocks"> instead.
-
-=head2 Range constraint
-
-The suggestion to use a C<Range> constraint on the C<count> field to limit
-the number of repetitions of a Repeatable block, has been withdrawn.
-
-This was only useful in the case that there were no initial rows to be edited,
-otherwise the C<max()> value could not be known ahead of time.
-
-See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
-
 =head1 CAVEATS
 
 To ensure your column's inflators and deflators are called, we have to 
@@ -1726,6 +1701,20 @@ Because of this, beware of having column names which clash with DBIx::Class
 built-in method-names, such as C<delete>. - It will have obviously 
 undesirable results!
 
+=head1 REMOVED METHODS
+
+=head2 new_empty_row
+
+See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
+
+=head2 new_empty_row_multi
+
+See C<new_rows_max> in L</"Config options for Repeatable blocks"> instead.
+
+=head2 Range constraint
+
+See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
+
 =head1 SUPPORT
 
 Project Page:
